package xyz.pixelatedw.finalbeta.mixin;

import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfoReturnable;

import net.minecraft.entity.LivingEntity;
import net.minecraft.entity.player.Player;
import net.minecraft.level.Level;
import net.minecraft.tile.DoorTile;
import net.minecraft.tile.Tile;
import net.minecraft.tile.material.Material;
import xyz.pixelatedw.finalbeta.ModConfig;
import xyz.pixelatedw.finalbeta.WyHelper;

@Mixin(DoorTile.class)
public class DoorTileMixin extends Tile {

	protected DoorTileMixin(int i, int j, Material arg) {
		super(i, j, arg);
	}

	/*
	 * This is horrible and with edge cases, not the best of fixes, needs more
	 * work preferably changing the metadata of doors to allow for more states.
	 * 
	 * When times get hard always remember:
	 * "If it's stupid but it works, it's still stupid but it works so fuck it."
	 */

	@Inject(method = "activate", at = @At("HEAD"), cancellable = true)
	public void activate(Level level, int x, int y, int z, Player player, CallbackInfoReturnable<Boolean> cir) {
		if (!ModConfig.FIX_DOUBLE_DOORS.get()) {
			return;
		}
		
		DoorTile tile = ((DoorTile) (Object) this);
		if (tile.material != Material.METAL) {
			int tileMeta = level.getTileMeta(x, y, z);
			int yOffset = 0;

			if ((tileMeta & 8) != 0) {
				setDoorTileMeta(level, x, y - 1, z, (tileMeta ^ 4) - 8);
				yOffset = 1;
			} else {
				setDoorTileMeta(level, x, y, z, tileMeta ^ 4);
			}

			boolean state = false;
			int hash = generatePosHash(x, y - yOffset, z);
			if (WyHelper.DOOR_STATES.containsKey(hash)) {
				state = WyHelper.DOOR_STATES.get(hash);
			}
			WyHelper.DOOR_STATES.put(hash, !state);
			cir.setReturnValue(true);
		}
	}
	
	@Inject(method = "method_837", at = @At("HEAD"), cancellable = true)
	public void updateRedstoneSignal(Level level, int x, int y, int z, boolean openSignal, CallbackInfo ci) {

		if (!ModConfig.FIX_DOUBLE_DOORS.get()) {
			return;
		}

		// Top Tile Meta
		// Open: 12 11 - 13 8 - 14 9 - 15 10
		// Close: 8 15 - 9 12 - 10 13 - 11 14

		// Bottom Tile Meta
		// Open: 4 3 - 5 0 - 6 1 - 7 2
		// Close: 0 7 - 1 4 - 2 5 - 3 6

		int hash = generatePosHash(x, y, z);
		if (WyHelper.DOOR_UPDATES.containsKey(hash)) {
			long lastUpdate = System.currentTimeMillis() - WyHelper.DOOR_UPDATES.get(hash);
			if (lastUpdate < 200) {
				ci.cancel();
				return;
			}
		}

		int tileMeta = level.getTileMeta(x, y, z);

		if (openSignal) {
			if (WyHelper.DOOR_STATES.containsKey(hash) && WyHelper.DOOR_STATES.get(hash)) {
				ci.cancel();
				return;
			}
			setDoorTileMeta(level, x, y, z, Math.floorMod((tileMeta + 4), 8));
			WyHelper.DOOR_STATES.put(hash, true);
		} else {
			if (WyHelper.DOOR_STATES.containsKey(hash) && !WyHelper.DOOR_STATES.get(hash)) {
				ci.cancel();
				return;
			}
			setDoorTileMeta(level, x, y, z, Math.floorMod((tileMeta - 4), 8));
			WyHelper.DOOR_STATES.put(hash, false);
		}

		ci.cancel();
	}
	
	@Override
	public void beforeDestroyedByExplosion(Level level, int x, int y, int z, int i1, float f) {
		int tileMeta = level.getTileMeta(x, y, z);
		int yOffset = 0;
		
		if ((tileMeta & 8) != 0) {
			yOffset = 1;
		}
		
		int hash = generatePosHash(x, y - yOffset, z);
		WyHelper.DOOR_UPDATES.remove(hash);
		WyHelper.DOOR_STATES.remove(hash);
		super.beforeDestroyedByExplosion(level, x, y, z, i1, f);
	}
	
	@Override
	public void afterPlaced(Level level, int x, int y, int z, LivingEntity entity) {
		int hash = generatePosHash(x, y, z);
		WyHelper.DOOR_STATES.put(hash, false);
	}

	private void setDoorTileMeta(Level level, int x, int y, int z, int meta) {
		int tileId = level.getTileId(x, y, z);
		level.setTileMeta(x, y, z, meta);
		generateAndSaveHash(x, y, z);
		if (level.getTileId(x, y + 1, z) == tileId) {
			level.setTileMeta(x, y + 1, z, meta + 8);
			generateAndSaveHash(x, y, z);
		}

		level.updateRedstone(x, y - 1, z, x, y, z);
		level.playLevelEvent((Player) null, 1003, x, y, z, 0);
	}

	public int generatePosHash(int x, int y, int z) {
		return (y * 31) + (z * 15) + x;
	}

	private void generateAndSaveHash(int x, int y, int z) {
		WyHelper.DOOR_UPDATES.put(generatePosHash(x, y, z), System.currentTimeMillis());
	}
}
